@@ -1,5 +1,4 @@
-# Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Chooser.pm,v 1.39 2008-03-31 21:10:17 nekral-guest Exp $
+# Locale::Po4a::Chooser -- Manage po4a modules
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -36,6 +35,8 @@ sub new {
         $modname = 'LaTeX';
     } elsif ($module eq 'bibtex') {
         $modname = 'BibTex';
+    } elsif ($module eq 'tex') {
+        $modname = 'TeX';
     } else {
         $modname = ucfirst($module);
     }
@@ -57,15 +58,15 @@ sub list {
     warn wrap_msg(gettext("List of valid formats:")
 #	."\n  - ".gettext("bibtex: BibTex bibliography format.")
 	."\n  - ".gettext("dia: uncompressed Dia diagrams.")
-	."\n  - ".gettext("docbook: Docbook XML.")
-	."\n  - ".gettext("guide: Gentoo Linux's xml documentation format.")
+	."\n  - ".gettext("docbook: DocBook XML.")
+	."\n  - ".gettext("guide: Gentoo Linux's XML documentation format.")
 #	."\n  - ".gettext("html: HTML documents (EXPERIMENTAL).")
-	."\n  - ".gettext("ini: .INI format.")
+	."\n  - ".gettext("ini: INI format.")
 	."\n  - ".gettext("kernelhelp: Help messages of each kernel compilation option.")
 	."\n  - ".gettext("latex: LaTeX format.")
 	."\n  - ".gettext("man: Good old manual page format.")
 	."\n  - ".gettext("pod: Perl Online Documentation format.")
-	."\n  - ".gettext("sgml: either debiandoc or docbook DTD.")
+	."\n  - ".gettext("sgml: either DebianDoc or DocBook DTD.")
 	."\n  - ".gettext("texinfo: The info page format.")
 	."\n  - ".gettext("tex: generic TeX documents (see also latex).")
 	."\n  - ".gettext("text: simple text document.")
@@ -82,9 +83,11 @@ sub list {
 1;
 __END__
 
+=encoding UTF-8
+
 =head1 NAME
 
-Locale::Po4a::Chooser - Manage po4a modules
+Locale::Po4a::Chooser - manage po4a modules
 
 =head1 DESCRIPTION
 
@@ -114,17 +117,20 @@ L<Locale::Po4a::Po(3pm)>
 L<Locale::Po4a::Dia(3pm)>,
 L<Locale::Po4a::Docbook(3pm)>,
 L<Locale::Po4a::Guide(3pm)>,
+L<Locale::Po4a::Halibut(3pm)>,
 L<Locale::Po4a::Ini(3pm)>,
 L<Locale::Po4a::KernelHelp(3pm)>,
 L<Locale::Po4a::LaTeX(3pm)>,
 L<Locale::Po4a::Man(3pm)>,
 L<Locale::Po4a::Pod(3pm)>,
 L<Locale::Po4a::Sgml(3pm)>,
+L<Locale::Po4a::TeX(3pm)>,
 L<Locale::Po4a::Texinfo(3pm)>,
 L<Locale::Po4a::Text(3pm)>,
 L<Locale::Po4a::Wml(3pm)>.
 L<Locale::Po4a::Xhtml(3pm)>,
-L<Locale::Po4a::Xml(3pm)>.
+L<Locale::Po4a::Xml(3pm)>,
+L<Locale::Po4a::Wml(3pm)>.
 
 =back
 
