@@ -21,13 +21,15 @@
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
-Locale::Po4a::LaTeX - Convert LaTeX documents and derivates from/to PO files
+Locale::Po4a::LaTeX - convert LaTeX documents and derivates from/to PO files
 
 =head1 DESCRIPTION
 
-The po4a (po for anything) project goal is to ease translations (and more
+The po4a (PO for anything) project goal is to ease translations (and more
 interestingly, the maintenance of translations) using gettext tools on
 areas where they were not expected like documentation.
 
@@ -307,7 +309,7 @@ register_generic_command("*textcolor,{}{_}");
 register_generic_command("*colorbox,{}{_}");
 register_generic_command("*fcolorbox,{}{}{_}");
 register_generic_command("*pagecolor,{_}");
-register_generic_command("*color,{_}");
+register_generic_command("*color,{}");
 
 # equations/theorems
 register_generic_command("*qedhere,");
