@@ -1,12 +1,14 @@
 #!/usr/bin/perl -w
 
+=encoding UTF-8
+
 =head1 NAME
 
-Locale::Po4a::Man - Convert manual pages from/to PO files
+Locale::Po4a::Man - convert manual pages from/to PO files
 
 =head1 DESCRIPTION
 
-The po4a (po for anything) project goal is to ease translations (and more
+The po4a (PO for anything) project goal is to ease translations (and more
 interestingly, the maintenance of translations) using gettext tools on
 areas where they were not expected like documentation.
 
@@ -26,10 +28,7 @@ that translators can't mess up with them.
 Unindented paragraphs are automatically rewrapped for the translator.  This
 can lead to some minor difference in the generated output, since the
 rewrapping rules used by groff aren't very clear. For example, two spaces
-after a parenthesis are sometimes preserved, while typographic rules only
-ask to preserve the two spaces after the period sign (ok, I'm not native
-speaker, and I'm not sure of that. If you have any other information,
-you're welcome). 
+after a parenthesis are sometimes preserved.
 
 Anyway, the difference will only be about the position of the extra spaces
 in wrapped paragraph, and I think it's worth.
@@ -39,7 +38,7 @@ in wrapped paragraph, and I think it's worth.
 The first change is about font change specifications.  In nroff, there are
 several ways to specify if a given word should be written in small, bold or
 italics. In the text to translate, there is only one way, borrowed from the
-pod (perl online documentation) format:
+POD (Perl online documentation) format:
 
 =over
 
@@ -101,7 +100,7 @@ character (i.e., using `\&` or '\&' respectively).
 
 Since these chars are used to delimit parts under font modification, you
 can't use them verbatim. Use EE<lt>ltE<gt> and EE<lt>gtE<gt> instead (as in
-pod, one more time).
+POD, one more time).
 
 =head1 OPTIONS ACCEPTED BY THIS MODULE
 
@@ -324,15 +323,15 @@ it's valid in nroff, but would complicate too much the parser to be
 handled.
 
 Of course, another possibility is to use another format, more translator
-friendly (like pod using po4a::pod, or one of the xml familly like sgml),
+friendly (like POD using po4a::pod, or one of the XML familly like SGML),
 but thanks to po4a::man it isn't needed anymore. That being said, if the
-source format of your documentation is pod, or xml, it may be clever to
+source format of your documentation is POD, or XML, it may be clever to
 translate the source format and not this generated one. In most cases,
 po4a::man will detect generated pages and issue a warning. It will even
-refuse to process Pod generated pages, because those pages are perfectly
+refuse to process POD generated pages, because those pages are perfectly
 handled by po4a::pod, and because their nroff counterpart defines a lot of
 new macros I didn't want to write support for. On my box, 1432 of the 4323
-pages are generated from pod and will be ignored by po4a::man.
+pages are generated from POD and will be ignored by po4a::man.
 
 In most cases, po4a::man will detect the problem and refuse to process the
 page, issuing an adapted message. In some rare cases, the program will
@@ -351,7 +350,7 @@ Some tests are regularly run on Linux boxes:
 =item *
 
 one third of the pages are refused because they were generated from
-another format supported by po4a (e.g. pod or SGML).
+another format supported by po4a (e.g. POD or SGML).
 
 =item *
 
@@ -651,7 +650,7 @@ NEW_LINE:
         unless ($allow_generated) {
                 # Check for comments indicating that the file was generated.
                 if ($c =~ /Pod::Man/) {
-                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
+                    warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the POD file with the pod module of po4a."));
                     exit 254;
                 } elsif ($c =~ /generated by help2man/)    {
                     warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
@@ -707,7 +706,7 @@ NEW_LINE:
         @next_comments = ();
     }
 
-    # A .I or .B request chnage the current font
+    # A .I or .B request change the current font
     # and on exit, switch the font to Roman
     # When one of these request doesn't have its argument on its line
     # (and when we support this usage), we must keep this font request to
@@ -726,7 +725,7 @@ NEW_LINE:
                 $insert_font = "\\f$font$insert_font";
                 $line = $l2;
                 $ref = $r2;
-            } elsif ($l2 =~ /^[.'][\t ]*(SH|TP|P|PP|LP)(?:[\t ]|\s*$)/) {
+            } elsif ($l2 =~ /^[.'][\t ]*(SH|TP|TQ|P|PP|LP)(?:[\t ]|\s*$)/) {
                 $line =~ s/^\.([BI])\s*$/$insert_font\\f$1/;
                 $self->SUPER::unshiftline($l2,$r2);
             } elsif ($l2 =~ /^([.'][\t ]*(?:IP)[\t ]+"?)(.*)$/) {
@@ -1049,7 +1048,7 @@ sub post_trans {
                     )?
                     [.']
                    )/\\&$1/mgx;
-    } elsif ($self->{type} =~ m/^(TP)$/) {
+    } elsif ($self->{type} =~ m/^(TP|TQ)$/) {
         # But it is also needed for some type (e.g. TP, if followed by a
         # font macro)
         # This regular expression is the same as above
@@ -1063,7 +1062,7 @@ sub post_trans {
     $str =~ s/\Q$nbs_out/\\ /sg if defined $nbs_out;
     # No nbsp (said "\ " in groff on the last pos of the line, or groff adds
     # an extra space
-    $str =~ s/\\ \n/\\ /sg;
+    $str =~ s/\\ \n(?=.)/\\ /sg;
 
     # Make sure we compute internal sequences right.
     # think about: B<AZE E<lt> EZA E<gt>>
@@ -1156,7 +1155,7 @@ sub post_trans {
         $str = $tmp.$str;
     }
     if (not defined $self->{type}) {
-        $str =~ s/ $//mg;
+        $str =~ s/(?<!\\) $//mg;
     }
 
     print STDERR "$str\n" if ($debug{'postrans'});
@@ -1636,7 +1635,7 @@ sub splitargs {
                 $elem =~ s/^3/B/;
                 $elem =~ s/^4/(BI/;
 
-            if ($elem =~ /^([1-4]|B|I|R|\(CW|\[\]|\[P\]|L)(.*)$/s) {
+            if ($elem =~ /^([1-4]|B|I|R|\(..|\[[^]]*\]|L)(.*)$/s) {
                 # Each element should now start by a recognized font modifier
                 my $new_font = $1;
                 my $arg = $2;
@@ -1844,10 +1843,7 @@ $macro{'LP'}=$macro{'P'}=$macro{'PP'}=sub {
 $macro{'RE'}=\&noarg;
 $macro{'RS'}=\&untranslated;
 
-#Indented Paragraph Macros
-#  .TP i    Begin  paragraph  with  hanging tag.  The tag is given on the next line,
-#           but its results are like those of the .IP command.
-$macro{'TP'}=sub {
+sub parse_tp_tq {
     my $self=shift;
     my ($line,$l2,$ref2);
     $line .= $_[0] if defined($_[0]);
@@ -1880,6 +1876,13 @@ $macro{'TP'}=sub {
     } else {
 	$self->pushline($self->t($l2, "wrap" => 0)."\n");
     }
+}
+
+#Indented Paragraph Macros
+#  .TP i    Begin  paragraph  with  hanging tag.  The tag is given on the next line,
+#           but its results are like those of the .IP command.
+$macro{'TP'}=sub {
+    parse_tp_tq(@_);
 
     # From info groff:
     # Note that neither font shape nor font size of the label [i.e. argument
@@ -1888,6 +1891,16 @@ $macro{'TP'}=sub {
     set_font("R");
 };
 
+# Indented Paragraph Macros
+# .TQ  Indicates continuation of the .TP labels that precede the indented
+#      paragraph.
+$macro{'TQ'}=sub {
+    warn "Macro $_[1] does not accept any argument\n"
+	if (defined ($_[2]));
+
+    parse_tp_tq(@_);
+};
+
 #   Indented Paragraph Macros
 #       .HP i    Begin paragraph with a hanging indent (the first line of  the  paragraph
 #                is  at  the  left margin of normal paragraphs, and the rest of the para-
@@ -2455,6 +2468,14 @@ sub define_mdoc_macros {
     $macro{'In'} = \&translate_mdoc;
     # NetBSD Macro
     $inline{'Nx'} = 1;
+    # Curly brackets
+    $inline{'Brq'} = 1;
+    # Corporate name
+    $inline{'%Q'} = 1;
+    # Math symbol
+    $inline{'Ms'} = 1;
+    # Prints 'under development'
+    $inline{'Ud'} = 1;
 
     # This macro is a groff macro. I don't know if ot is valid in an mdoc page.
     # But this is used in some pages and seems to work
