@@ -28,13 +28,15 @@
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
-Locale::Po4a::Xml - Convert XML documents and derivates from/to PO files
+Locale::Po4a::Xml - convert XML documents and derivates from/to PO files
 
 =head1 DESCRIPTION
 
-The po4a (po for anything) project goal is to ease translations (and more
+The po4a (PO for anything) project goal is to ease translations (and more
 interestingly, the maintenance of translations) using gettext tools on
 areas where they were not expected like documentation.
 
@@ -229,7 +231,7 @@ written in most XML based documents.
 There are some options (described in the next section) that can customize
 this behavior.  If this doesn't fit to your document format you're encouraged
 to write your own module derived from this, to describe your format's details.
-See the section "Writing derivate modules" below, for the process description.
+See the section B<WRITING DERIVATE MODULES> below, for the process description.
 
 =cut
 
@@ -285,7 +287,7 @@ documents.
 =item B<ontagerror>
 
 This option defines the behavior of the module when it encounter a invalid
-Xml syntax (a closing tag which does not match the last opening tag, or a
+XML syntax (a closing tag which does not match the last opening tag, or a
 tag's attribute without value).
 It can take the following values:
 
@@ -322,6 +324,12 @@ String that will try to match with the first line of the document's doctype
 (if defined). If it doesn't, a warning will indicate that the document
 might be of a bad type.
 
+=item B<addlang>
+
+String indicating the path (e.g. E<lt>bbbE<gt>E<lt>aaaE<gt>) of a tag
+where a lang="..." attribute shall be added. The language will be defined
+as the basename of the PO file without any .po extension.
+
 =item B<tags>
 
 Space-separated list of tags you want to translate or skip.  By default,
@@ -356,6 +364,11 @@ Instead, replace all attributes of a tag by po4a-id=<id>.
 This is useful when attributes shall not be translated, as this simplifies the
 strings for translators, and avoids typos.
 
+=item B<customtag>
+
+Space-separated list of tags which should not be treated as tags.
+These tags are treated as inline, and do not need to be closed.
+
 =item B<break>
 
 Space-separated list of tags which should break the sequence.
@@ -480,6 +493,7 @@ sub initialize {
 	              'folded_attributes' => \%folded_attributes);
 	@save_holders = (\%holder);
 
+	$self->{options}{'addlang'}=0;
 	$self->{options}{'nostrip'}=0;
 	$self->{options}{'wrap'}=0;
 	$self->{options}{'caseinsensitive'}=0;
@@ -493,6 +507,7 @@ sub initialize {
 	$self->{options}{'foldattributes'}=0;
 	$self->{options}{'inline'}='';
 	$self->{options}{'placeholder'}='';
+	$self->{options}{'customtag'}='';
 	$self->{options}{'doctype'}='';
 	$self->{options}{'nodefault'}='';
 	$self->{options}{'includeexternal'}=0;
@@ -517,6 +532,7 @@ sub initialize {
 	$self->{options}{'_default_inline'}='';
 	$self->{options}{'_default_placeholder'}='';
 	$self->{options}{'_default_attributes'}='';
+	$self->{options}{'_default_customtag'}='';
 
 	#It will maintain the list of the translatable tags
 	$self->{tags}=();
@@ -530,6 +546,8 @@ sub initialize {
 	$self->{inline}=();
 	#It will maintain the list of the placeholder tags
 	$self->{placeholder}=();
+	#It will maintain the list of the customtag tags
+	$self->{customtag}=();
 	#list of the tags that must not be set in the tags or inline category
 	#by this module or sub-module (unless specified in an option)
 	$self->{nodefault}=();
@@ -646,15 +664,15 @@ define a tag type you'll have to make a hash with the following keys:
 
 =over 4
 
-=item beginning
+=item B<beginning>
 
 Specifies the beginning of the tag, after the "E<lt>".
 
-=item end
+=item B<end>
 
 Specifies the end of the tag, before the "E<gt>".
 
-=item breaking
+=item B<breaking>
 
 It says if this is a breaking tag class.  A non-breaking (inline) tag is one
 that can be taken as part of the content of another tag.  It can take the
@@ -662,12 +680,12 @@ values false (0), true (1) or undefined.  If you leave this undefined, you'll
 have to define the f_breaking function that will say whether a concrete tag of
 this class is a breaking tag or not.
 
-=item f_breaking
+=item B<f_breaking>
 
 It's a function that will tell if the next tag is a breaking one or not.  It
-should be defined if the "breaking" option is not.
+should be defined if the B<breaking> option is not.
 
-=item f_extract
+=item B<f_extract>
 
 If you leave this key undefined, the generic extraction function will have to
 extract the tag itself.  It's useful for tags that can have other tags or
@@ -675,7 +693,7 @@ special structures in them, so that the main parser doesn't get mad.  This
 function receives a boolean that says if the tag should be removed from the
 input stream or not.
 
-=item f_translate
+=item B<f_translate>
 
 This function receives the tag (in the get_string_until() format) and returns
 the translated tag (translated attributes or all needed transformations) as a
@@ -936,6 +954,13 @@ sub tag_trans_open {
 
 	$name = $self->treat_attributes(@tag);
 
+	if (defined $self->{options}{'addlang'}) {
+		my $struct = $self->get_path();
+		if ($struct eq $self->{options}{'addlang'}) {
+			$name .= ' lang="'.$self->{TT}{po_in}->{lang}.'"';
+		}
+	}
+
 	return $name;
 }
 
@@ -1238,6 +1263,7 @@ sub treat_attributes {
 #   W: the content shall not be re-wrapped
 #   i: the tag shall be inlined
 #   p: a placeholder shall replace the tag (and its content)
+#   n: a custom tag
 #
 # A translatable inline tag in an untranslated tag is treated as a translatable breaking tag.
 my %translate_options_cache;
@@ -1309,6 +1335,12 @@ sub get_translate_options {
 		}
 	}
 
+	$tag = $self->get_tag_from_list($path, $self->{customtag});
+	if (defined $tag) {
+		$usedefault = 0;
+		$options = "in"; # This erase any other setting
+	}
+
 	if ($usedefault) {
 		$options = $self->{options}{'defaulttranslateoption'};
 	}
@@ -1411,7 +1443,7 @@ sub treat_content {
 						my @sub_translations = ();
 						my %folded_attributes;
 						my %new_holder = ('paragraph' => \@new_paragraph,
-						                  'open' => $text[0],
+						                  'open' => $self->join_lines(@text),
 						                  'translation' => "",
 						                  'close' => undef,
 						                  'sub_translations' => \@sub_translations,
@@ -1437,7 +1469,9 @@ sub treat_content {
 							@text = ("<$cur_tag_name po4a-id=$id>", $tag_ref);
 						}
 					}
-					push @path, $cur_tag_name;
+					unless ($t_opts =~ m/n/) {
+						push @path, $cur_tag_name;
+					}
 				} elsif ($tag_types[$type]->{'beginning'} eq "/") {
 					# Closing inline tag
 
@@ -1469,8 +1503,8 @@ sub treat_content {
 						# the holder from the stack.
 						my $holder = pop @save_holders;
 						# We need to keep the translation of this holder
-						my $translation = $holder->{'open'}.$holder->{'translation'}.$text[0];
-						# FIXME: @text could be multilines.
+						my $translation = $holder->{'open'}.$holder->{'translation'};
+						$translation .= $self->join_lines(@text);
 
 						@text = ();
 
@@ -1701,6 +1735,8 @@ sub treat_options {
 		$self->{options}{'_default_inline'}       = lc $self->{options}{'_default_inline'};
 		$self->{options}{'placeholder'}           = lc $self->{options}{'placeholder'};
 		$self->{options}{'_default_placeholder'}  = lc $self->{options}{'_default_placeholder'};
+		$self->{options}{'customtag'}             = lc $self->{options}{'customtag'};
+		$self->{options}{'_default_customtag'}    = lc $self->{options}{'_default_customtag'};
 	}
 
 	$self->{options}{'nodefault'} =~ /^\s*(.*)\s*$/s;
@@ -1787,7 +1823,6 @@ sub treat_options {
 		}
 	}
 
-	my @list_inline;
 	$self->{options}{'inline'} =~ /^\s*(.*)\s*$/s;
 	foreach my $tag (split(/\s+/s,$1)) {
 		$tag =~ m/^(.*?)(<.*)$/;
@@ -1814,6 +1849,19 @@ sub treat_options {
 			       or defined $self->{placeholder}->{$2};
 	}
 
+	$self->{options}{'customtag'} =~ /^\s*(.*)\s*$/s;
+	foreach my $tag (split(/\s+/s,$1)) {
+		$tag =~ m/^(.*?)(<.*)$/;
+		$self->{customtag}->{$2} = $1 || "";
+	}
+	$self->{options}{'_default_customtag'} =~ /^\s*(.*)\s*$/s;
+	foreach my $tag (split(/\s+/s,$1)) {
+		$tag =~ m/^(.*?)(<.*)$/;
+		$self->{customtag}->{$2} = $1 || ""
+			unless    $list_nodefault{$2}
+			       or defined $self->{customtag}->{$2};
+	}
+
 	# There should be no translated and untranslated tags
 	foreach my $tag (keys %{$self->{translated}}) {
 		die wrap_mod("po4a::xml",
@@ -1821,7 +1869,7 @@ sub treat_options {
 		                      "Tag '%s' both in the %s and %s categories."), $tag, "translated", "untranslated")
 			if defined $self->{untranslated}->{$tag};
 	}
-	# There should be no inline, break, and placeholder tags
+	# There should be no inline, break, placeholder, and customtag tags
 	foreach my $tag (keys %{$self->{inline}}) {
 		die wrap_mod("po4a::xml",
 		             dgettext("po4a",
@@ -1831,12 +1879,26 @@ sub treat_options {
 		             dgettext("po4a",
 		                      "Tag '%s' both in the %s and %s categories."), $tag, "inline", "placeholder")
 			if defined $self->{placeholder}->{$tag};
+		die wrap_mod("po4a::xml",
+		             dgettext("po4a",
+		                      "Tag '%s' both in the %s and %s categories."), $tag, "inline", "customtag")
+			if defined $self->{customtag}->{$tag};
 	}
 	foreach my $tag (keys %{$self->{break}}) {
 		die wrap_mod("po4a::xml",
 		             dgettext("po4a",
 		                      "Tag '%s' both in the %s and %s categories."), $tag, "break", "placeholder")
 			if defined $self->{placeholder}->{$tag};
+		die wrap_mod("po4a::xml",
+		             dgettext("po4a",
+		                      "Tag '%s' both in the %s and %s categories."), $tag, "break", "customtag")
+			if defined $self->{customtag}->{$tag};
+	}
+	foreach my $tag (keys %{$self->{placeholder}}) {
+		die wrap_mod("po4a::xml",
+		             dgettext("po4a",
+		                      "Tag '%s' both in the %s and %s categories."), $tag, "placeholder", "customtag")
+			if defined $self->{customtag}->{$tag};
 	}
 }
 
@@ -1854,15 +1916,15 @@ The valid options are:
 
 =over 4
 
-=item include
+=item B<include>
 
 This makes the returned array to contain the searched text
 
-=item remove
+=item B<remove>
 
 This removes the returned stream from the input
 
-=item unquoted
+=item B<unquoted>
 
 This ensures that the searched text is outside any quotes
 
